import { EventEmitter, Component, Input, Output, ViewChild, ContentChild, Pipe, Injector, ElementRef, Renderer2, NgModule } from '@angular/core';
import { Direction, TreeNodeState, SvogvCoreModule } from '@svogv/core';
import { CommonModule } from '@angular/common';
import { ReactiveFormsModule, FormsModule } from '@angular/forms';

/**
 * The pagination component creates a few buttons to navigate a grid. The underlaying model
 * is going to handle the date on the client. The pagination does not support a server backend,
 * all relevant data must be loaded first.
 *
 * Example of usage:
 * @example
 * ```html
 * <ac-pagination></ac-pagination>
 * ```
 *
 * <example-url>/#/widget/grid</example-url>
 */
class DataGridPaginationComponent {
    constructor() {
        /**
         * An event fired once the user has changed the page by clicking a button.
         */
        this.pageNumberChanged = new EventEmitter();
        this.currentPageNumber = 1;
    }
    ngOnInit() {
        this.setCurrentPage(1);
    }
    ngOnChanges(changes) {
        if (changes['maxPageIndex']) {
            const change = changes['maxPageIndex'];
            if (this.currentPageNumber > change.currentValue) {
                // throws ExpressionChangedAfterItHasBeenCheckedException
                // if there's no setTimeout.
                // no need to add setTimeout if ngOnChanges
                // is fired after changes made on root component.
                setTimeout(() => this.setCurrentPage(1), 1);
            }
        }
    }
    setCurrentPage(pageNumber, event) {
        if (event) {
            event.preventDefault();
        }
        if (pageNumber === 0 || pageNumber > this.maxPageIndex
            || pageNumber === this.currentPageNumber) {
            return;
        }
        this.pageNumberChanged.emit(pageNumber);
        this.currentPageNumber = pageNumber;
    }
    range(min, max) {
        const result = new Array();
        for (let i = min; i <= max; i++) {
            result.push(i);
        }
        return result;
    }
    get pageStartNumber() {
        const startNumber = this.currentPageNumber <= 4
            ? 1
            : this.currentPageNumber >= this.maxPageIndex - 3
                ? this.maxPageIndex - 6
                : this.currentPageNumber - 3;
        return startNumber < 1 ? 1 : startNumber;
    }
    get pageEndNumber() {
        const pageEnd = this.pageStartNumber + 6;
        return pageEnd > this.maxPageIndex ? this.maxPageIndex : pageEnd;
    }
}
DataGridPaginationComponent.decorators = [
    { type: Component, args: [{
                selector: 'ac-datagrid-pagination',
                template: "<div>\n  <ul class=\"pagination float-right\" [ngClass]=\"{ 'pagination-sm': size == 'sm', 'pagination-lg': size == 'lg' }\">\n    <li [class.disabled]=\"currentPageNumber === 1 || !maxPageIndex\" class=\"page-item\">\n      <a href (click)=\"setCurrentPage(1, $event)\" aria-label=\"Previous\" class=\"page-link\">\n        <span aria-hidden=\"true\">\u00AB</span>\n      </a>\n    </li>\n    <li [class.disabled]=\"currentPageNumber === 1 || !maxPageIndex\" class=\"page-item\">\n      <a\n        href\n        aria-label=\"Previous\"\n        (click)=\"setCurrentPage(currentPageNumber - 1, $event)\"\n        class=\"page-link\">\n        <span aria-hidden=\"true\">\u2039</span>\n      </a>\n    </li>\n    <li\n      *ngFor=\"let index of range(pageStartNumber, pageEndNumber)\"\n      [class.active]=\"currentPageNumber === index\"\n      class=\"page-item\">\n      <a href (click)=\"setCurrentPage(index, $event)\" class=\"page-link\">\n        <span aria-hidden=\"true\">{{ index }}</span>\n      </a>\n    </li>\n    <li [class.disabled]=\"currentPageNumber === maxPageIndex || !maxPageIndex\">\n      <a class=\"page-link\"\n        href\n        (click)=\"setCurrentPage(currentPageNumber + 1, $event)\"\n        aria-label=\"Last\"\n      >\n        <span aria-hidden=\"true\">\u203A</span>\n      </a>\n    </li>\n    <li [class.disabled]=\"currentPageNumber === maxPageIndex || !maxPageIndex\" class=\"page-item\">\n      <a href (click)=\"setCurrentPage(maxPageIndex, $event)\" aria-label=\"Last\" class=\"page-link\">\n        <span aria-hidden=\"true\">\u00BB</span>\n      </a>\n    </li>\n  </ul>\n</div>\n",
                styles: [""]
            },] }
];
DataGridPaginationComponent.propDecorators = {
    maxPageIndex: [{ type: Input }],
    pageNumberChanged: [{ type: Output }],
    size: [{ type: Input }]
};

/**
 * A classic data grid. You provide a model to handle all features. The model is build from a
 * simple array of objects with decorators.
 *
 * > See 'Documentation and Examples' tab for a complete documentation.
 *
 * ### Summary
 *
 * The datagrid provides basic functions for data tables:
 *
 * * sorting
 * * filtering
 * * pagination
 * * editing
 *
 * Provide an decorator enhanced model and the grid appears driven by model meta data.
 *
 * There are many attributes and ways to change the appearance. Also some classes can be controlled by
 * the host component:
 *
 * * `.col-borders`
 * * `.col-last`
 * * `.col-first`
 *
 * All these styles are applied to the <col> elements of the underlying table.
 *
 * The model used in the example is an array of objects, where the properties are decorated with
 * various decorators used to control the grid's render behavior.
 *
 * ~~~typescript
 * const data: UserViewModel[] = this.dataSource; // provide a simple array here
 *  this.model = new DataGridModel<UserViewModel>(data, UserViewModel);
 * ~~~
 *
 * The class {@link DataGridModel} controls the grid. You must provide a viewmodel, this is mandatory. The viewmodel is being examined at runtime,
 * so assure you provide a class and set all properties to a default to force creation of properties.
 *
 * ~~~typescript
 * export class UserViewModel {
 *  email: string = ''; // the = '' is necessary!
 *  // more omitted for brevity
 * }
 * ~~~
 *
 * The grid can be extended with the {@link DataGridPaginationComponent} to page through huge data sets. The model
 * handles the pagination, the additional {@link DataGridPaginationComponent} is only a predefined renderer that
 * supports the used theme.
 *
 * <example-url>/#/widget/grid</example-url>
 *
 * @example
 *  <ac-datagrid
 *      [model]="model"
 *      [showActions]="false"
 *      [columnStyle]=""
 *  ></ac-datagrid>
 */
class DataGridComponent {
    constructor() {
        /**
         * @ignore
         */
        this.directionEnumHelper = Direction;
        this.externals = {};
        /**
         * Show the action column at all. Use {@link showDeleteButton and  @see showEditButton to switch the}buttons
         * on or off individually. Default is `true` (actions visible).
         */
        this.showActions = true;
        /**
         * The text that appears on the Delete button. Default is 'Delete'.
         */
        this.textDeleteButton = 'Delete';
        /**
         * The text that appears on the Edit button. Default is 'Edit'.
         */
        this.textEditButton = 'Edit';
        /**
         * The column header of the column that shows the buttons. Default is 'Actions'.
         */
        this.textButtonsHeader = 'Actions';
        /**
         * The text that appears if there are no items to show. Can also be overwritten by a more complex piece
         * of code by adding a template like this:
         *
         * @example
         * <ng-template #data-warning-noitems>
         *   <div class="alert alert-danger">The grid is empty</div>
         * </ng-template>
         */
        this.textNoItems = 'There are no items to show';
        /**
         * Event forwarded from model class and being fired after the model class's onEdit event.
         * The event is invoked by the appropriate internal button via click.
         */
        this.editItem = new EventEmitter();
        /**
         * Event forwarded from model class and being fired after the model class's onDelete event.
         * The event is invoked by the appropriate internal button via click.
         */
        this.deleteItem = new EventEmitter();
        // tslint:disable-next-line:member-ordering
        /**
         * @ignore
         */
        this.warnProp = {};
    }
    /**
     * The filter value to filter the content. The data is of type `{ [prop: string]: any }`.
     *
     * @param value A dictionary with filter instructions as shown below. The filter logic applies one after another, like an __AND__ conjunction.
     *
     * @example
     * {
     *    "email": "paul@sample.com"
     *    "name": "Paul"
     * }
     *
     */
    set filter(value) {
        if (this.model) {
            this.model.searchValue = value;
        }
    }
    ngAfterViewInit() {
        if (this.model) {
            this.model.onEdit.subscribe(item => this.editItem.emit(item));
            this.model.onDelete.subscribe(item => this.deleteItem.emit(item));
        }
    }
    ngOnDestroy() {
        if (this.model) {
            this.model.onEdit.unsubscribe();
            this.model.onDelete.unsubscribe();
        }
    }
    /**
     * @ignore
     * Controls the template used to display certain data types.
     * If the host provides a template it's being used, otherwise a fallback is provided
     * @param uiHint Property of {@link UiHint} decorator
     */
    getActiveTemplate(uiHint, prop) {
        if (this[uiHint]) {
            // if provided by user via ContentChild and overwriting defaults (string == string etc.)
            return this[uiHint];
        }
        if (this.externals[uiHint]) {
            // if provided by user via ContentChild but completely replaced
            return this.externals[uiHint];
        }
        if (this[`${uiHint}Fallback`]) {
            // otherwise we take ours from ng-template via ViewChild
            return this[`${uiHint}Fallback`];
        }
        // if we go here the model requested a custom template that didn't exists
        if (this.warnProp[prop] !== uiHint) {
            console.warn(`Property ${prop} requested the template ${uiHint}, but it wasn't provided. Falling back to "string".`);
            this.warnProp[prop] = uiHint;
        }
        return this.stringFallback;
    }
}
DataGridComponent.decorators = [
    { type: Component, args: [{
                selector: 'ac-datagrid',
                template: "<div class=\"row\" *ngIf=\"model && model.headers\">\n  <div class=\"col-md-12\">\n    <table class=\"table table-condensed\">\n      <colgroup class=\"colborders\" [ngStyle]=\"columnStyle ? columnStyle.group : null\">\n        <col class=\"first\" [ngStyle]=\"columnStyle ? columnStyle.first : null\" />\n        <col class=\"middle\" [ngStyle]=\"columnStyle ? columnStyle.middle : null\" />\n        <col class=\"middle\" [ngStyle]=\"columnStyle ? columnStyle.middle : null\" />\n        <col class=\"middle\" [ngStyle]=\"columnStyle ? columnStyle.middle : null\" />\n        <col class=\"last\" [ngStyle]=\"columnStyle ? columnStyle.last : null\" />\n      </colgroup>\n      <thead>\n        <tr [ngStyle]=\"columnStyle ? columnStyle.header : null\" >\n          <th *ngFor=\"let header of model.headers\"\n              [ngClass]=\"{ 'sortableHeader': header.isSortable }\"\n              [ngStyle]=\"header.uiHint || {}\"\n              [title]=\"header.desc\"\n              (click)=\"header.isSortable ? model.sortColumn(header.prop, null, header.sortCallback) : null\">\n            <span class=\"headerText\">{{ header.text }}</span>\n            <span class=\"sort\" [ngStyle]=\"columnStyle ? columnStyle.headerButton : null\" *ngIf=\"header.isSortable\">\n              <button (click)=\"model.sortColumn(header.prop, directionEnumHelper.Ascending, header.sortCallback)\"\n                      class=\"up\"\n                      [hidden]=\"model.sortDirection[header.prop] === directionEnumHelper.Descending\"\n                      [ngStyle]=\"columnStyle ? columnStyle.headerSortButton : null\">\n                <i class=\"fa fa-sort-asc\"></i>\n              </button>\n              <button (click)=\"model.sortColumn(header.prop, directionEnumHelper.Descending, header.sortCallback)\"\n                      class=\"down\"\n                      [hidden]=\"model.sortDirection[header.prop] === directionEnumHelper.Ascending\"\n                      [ngStyle]=\"columnStyle ? columnStyle.headerSortButton : null\">\n                <i class=\"fa fa-sort-desc\"></i>\n              </button>\n            </span>\n          </th>\n          <th *ngIf=\"showActions\">\n            {{ textButtonsHeader }}\n          </th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr *ngIf=\"!model.totalFilteredRows\">\n          <td [attr.colspan]=\"model.headers.length\" class=\"text-center\">\n            <div class=\"alert alert-warning\">\n              <span #warningNoItems>\n                <ng-content select=\"[data-warning-noitems]\"></ng-content>\n              </span>\n              <ng-container *ngIf=\"!warningNoItems.innerHTML.trim()\">{{  textNoItems }}</ng-container>\n            </div>\n          </td>\n        </tr>\n        <tr *ngFor=\"let item of model.itemsOnCurrentPage\">\n          <td *ngFor=\"let header of model.headers\">\n            <ng-container\n              [ngTemplateOutlet]=\"getActiveTemplate(header.templateHint, header.prop)\"\n              [ngTemplateOutletContext]=\"{ $implicit: item[header.prop], pipe: header.pipe, params: header.pipeParams }\"\n            ></ng-container>\n          </td>\n          <td *ngIf=\"showActions\" [ngStyle]=\"columnStyle ? columnStyle.actionCell : null\" >\n            <button class=\"btn btn-primary btn-sm\" type=\"button\" (click)=\"model.editItem(item)\">\n              {{ textEditButton }}\n            </button>\n            <button class=\"btn btn-danger btn-sm\" type=\"button\" (click)=\"model.deleteItem(item)\">\n              {{ textDeleteButton }}\n            </button>\n          </td>\n        </tr>\n      </tbody>\n      <tfoot>\n        <tr [ngStyle]=\"columnStyle ? columnStyle.footer : null\" >\n          <td [attr.colspan]=\"model.headers.length + (showActions ? 1 : 0)\">\n            <div #footerRef>\n              <ng-content select=\"[data-footer]\"></ng-content>\n            </div>\n            <ng-container *ngIf=\"!footerRef.innerHTML.trim()\">&copy;</ng-container>\n          </td>\n        </tr>\n      </tfoot>\n    </table>\n  </div>\n</div>\n<ng-template #string let-data let-modelpipe=\"pipe\" let-params=\"params\">\n  {{ data | formatData: modelpipe: params }}\n</ng-template>\n<ng-template #boolean let-data>\n  <input type=\"checkbox\" disabled [checked]=\"data\" />\n</ng-template>\n<ng-template #date let-data let-modelpipe=\"pipe\" let-params=\"params\">\n  {{ data | formatData: modelpipe: params }}\n</ng-template>\n<ng-template #enum let-data>\n  {{ data }}\n</ng-template>\n<ng-template #number let-data let-modelpipe=\"pipe\" let-params=\"params\">\n  {{ data | formatData: modelpipe: params }}\n</ng-template>\n",
                styles: [".colborders col{border-right:1px solid azure}col.last{border-right:none!important}col.first{background-color:#eee}th{background:none}th.sortableHeader{cursor:pointer}.headerText{display:inline-block;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.sort{float:right}.sort button{background-color:transparent!important;border:0;border-radius:0;cursor:pointer;height:16px;padding:1px;width:16px}.sort button.up{left:0}.sort button.down{left:-16px}.rearrange{background-color:transparent;border:none;float:right;height:16px;margin:2px;width:16px}"]
            },] }
];
DataGridComponent.propDecorators = {
    stringFallback: [{ type: ViewChild, args: ['string', { static: true },] }],
    string: [{ type: ContentChild, args: ['string', { static: true },] }],
    booleanFallback: [{ type: ViewChild, args: ['boolean', { static: true },] }],
    boolean: [{ type: ContentChild, args: ['boolean', { static: true },] }],
    dateFallback: [{ type: ViewChild, args: ['date', { static: true },] }],
    date: [{ type: ContentChild, args: ['date', { static: true },] }],
    enumFallback: [{ type: ViewChild, args: ['enum', { static: true },] }],
    enum: [{ type: ContentChild, args: ['enum', { static: true },] }],
    numberFallback: [{ type: ViewChild, args: ['number', { static: true },] }],
    number: [{ type: ContentChild, args: ['number', { static: true },] }],
    externals: [{ type: Input }],
    columnStyle: [{ type: Input }],
    model: [{ type: Input }],
    showDeleteButton: [{ type: Input }],
    showEditButton: [{ type: Input }],
    showActions: [{ type: Input }],
    textDeleteButton: [{ type: Input }],
    textEditButton: [{ type: Input }],
    textButtonsHeader: [{ type: Input }],
    textNoItems: [{ type: Input }],
    filter: [{ type: Input }],
    reArrangeColumns: [{ type: Input }],
    editItem: [{ type: Output }],
    deleteItem: [{ type: Output }]
};

/**
 * This pipe allows the dynamic formatting of data. It is just for internal use.
 * The purpose is the forwarding of an externally defined pipe to the pipe formatter within the grid.
 */
class FormatDataPipe {
    constructor(injector) {
        this.injector = injector;
    }
    transform(value, pipeToken, pipeArgs) {
        if (!pipeToken) {
            console.log('FormatDataPipe:NoPipeToken');
            return value;
        }
        else {
            console.log('FormatDataPipe:PipeToken:', pipeToken);
            const pipe = this.injector.get(pipeToken);
            return pipe.transform(value, ...pipeArgs || []);
        }
    }
}
FormatDataPipe.decorators = [
    { type: Pipe, args: [{
                name: 'formatData'
            },] }
];
FormatDataPipe.ctorParameters = () => [
    { type: Injector }
];

/**
 * The Editor Widget. Creates a field with all required validators using decorators and forms service.
 */
class EditorComponent {
    constructor() {
        this.params = new Array();
    }
    getParams(key) {
        return this.first(this.params.filter((e) => e.key === key), 0);
    }
    first(array, n) {
        if (array == null) {
            return void 0;
        }
        if (n == null) {
            return array[0];
        }
        if (n < 0) {
            return [];
        }
        return array.slice(0, n);
    }
    ngOnInit() {
        this.model.formGroup.valueChanges.subscribe((data) => this.onValueChanged(data));
        // this is set by FormValidatorService
        const editorModel = this.model.formGroup.__editorModel__;
        // get type from form
        if (editorModel) {
            // get elementary types, this might get overwritten later according to decorators found
            if (typeof editorModel[this.model.name] === 'string') {
                this.model.type = 'text';
            }
            if (typeof editorModel[this.model.name] === 'boolean') {
                this.model.type = 'boolean';
            }
            if (typeof editorModel[this.model.name] === 'number') {
                this.model.type = 'number';
            }
            if (editorModel[this.model.name] instanceof Date) {
                this.model.type = 'calendar';
            }
            // make an instance to read the properties
            this.model.label = editorModel[`__displayName__${this.model.name}`] || this.model.label || this.model.name;
            this.model.tooltip = editorModel[`__displayDesc__${this.model.name}`] || this.model.tooltip || this.model.name;
            // render as range id there is a range definition
            if (editorModel[`__hasRangeFrom__${this.model.name}`] && Number(editorModel[`__hasRangeFrom__${this.model.name}`])) {
                this.model.fromValue = editorModel[`__hasRangeFrom__${this.model.name}`];
                this.model.type = 'range';
            }
            if (editorModel[`__hasRangeTo__${this.model.name}`] && Number(editorModel[`__hasRangeTo__${this.model.name}`])) {
                this.model.toValue = editorModel[`__hasRangeTo__${this.model.name}`];
                this.model.type = 'range';
            }
            // placeholder
            if (editorModel[`__hasWatermark__${this.model.name}`]) {
                this.model.waterMark = editorModel[`__watermark__${this.model.name}`];
            }
            // templates
            if (editorModel[`__hasTemplateHint__${this.model.name}`]) {
                this.model.type = editorModel[`__templatehint__${this.model.name}`].toLowerCase();
                if (editorModel[`__templatehintParams__${this.model.name}`]) {
                    this.params = editorModel[`__templatehintParams__${this.model.name}`];
                }
            }
            // render hidden fields as hidden even in forms
            if (editorModel[`__isHidden__${this.model.name}`]) {
                this.model.type = 'hidden';
            }
            // check readonly
            if (editorModel[`__isReadonly__${this.model.name}`]) {
                this.model.readonly = !!editorModel[`__isReadonly__${this.model.name}`];
            }
        }
    }
    onValueChanged(data) {
        // check validation on change
        this.errors = new Array();
        // tslint:disable-next-line:forin
        for (const error in this.model.formGroup.controls[this.model.name].errors) {
            this.errors.push(error);
        }
    }
}
EditorComponent.decorators = [
    { type: Component, args: [{
                selector: 'ac-editor',
                template: "<div class=\"form-group row\"\n    [formGroup]=\"model.formGroup\"\n     *ngIf=\"model.type != 'hidden' && model.formGroup\"\n     [ngClass]=\"{ 'has-danger': !model.formGroup.controls[name].valid && model.formGroup.controls[name].touched }\">\n  <label class=\"col-form-label col-md-3 col-sm-10\"\n    [attr.for]=\"name\"\n    [attr.title]=\"model.tooltip\" >{{ model.label }}{{ model.labelDivider }} </label>\n  <div [ngClass]=\"{ 'col-md-7 col-sm-10': model.inline }\">\n    <textarea *ngIf=\"model.type == 'textarea'\"\n            class=\"form-control\"\n            [id]=\"name\"\n            [readOnly]='model.readonly'\n            [formControlName]=\"name\"\n            [attr.rows]=\"getParams('rows')\"\n            [attr.cols]=\"getParams('cols')\">\n    </textarea>\n    <select *ngIf=\"model.type == 'enum' && model.enumValues\" class=\"form-control\" [id]=\"name\"\n          [formControlName]=\"name\" [disabled]='model.readonly'>\n      <option *ngFor=\"let option of model.enumValues\" [value]=\"option.key\">{{option.val}}</option>\n    </select>\n    <select *ngIf=\"model.type == 'list' && model.listValues\" class=\"form-control\" [id]=\"name\"\n          [formControlName]=\"name\" [disabled]='model.readonly'>\n      <option *ngFor=\"let option of model.listValues\" [value]=\"option.key\">{{option.val}}</option>\n    </select>\n    <input *ngIf=\"model.type == 'range'\" type=\"range\"\n        class=\"form-control\"\n        [placeholder]=\"model.waterMark\"\n        [attr.minvalue]=\"model.fromValue\"\n        [attr.maxvalue]=\"model.toValue\"\n        [attr.min]=\"model.fromValue\"\n        [attr.max]=\"model.toValue\"\n        [id]=\"name\"\n        [formControlName]=\"name\"\n        [title]=\"model.tooltip\" />\n    <input *ngIf=\"model.type == 'calendar'\" type=\"date\"\n        class=\"form-control\"\n        [placeholder]=\"model.waterMark\"\n        [readOnly]='model.readonly'\n        [id]=\"name\"\n        [formControlName]=\"name\"\n        [title]=\"model.tooltip\" />\n    <input *ngIf=\"model.type == 'number'\" type=\"number\"\n        class=\"form-control\"\n        [placeholder]=\"model.waterMark\"\n        [readOnly]='model.readonly'\n        [id]=\"name\"\n        [formControlName]=\"name\"\n        [title]=\"model.tooltip\" />\n    <input *ngIf=\"model.type == 'boolean'\" type=\"checkbox\"\n        class=\"form-control\"\n        [readOnly]='model.readonly'\n        [id]=\"name\"\n        [formControlName]=\"name\"\n        [title]=\"model.tooltip\" />\n    <label *ngIf=\"model.type == 'boolean'\" [attr.for]=\"name\"></label>\n    <input *ngIf=\"model.type == 'text' || model.type == ''\" [placeholder]=\"model.waterMark\"\n        [readOnly]='model.readonly'\n        type=\"text\" class=\"form-control\" [id]=\"name\" [formControlName]=\"name\" />\n    <ng-container *ngIf=\"model.type == 'template'\">\n        <ng-content></ng-content>\n    </ng-container>\n    <span class=\"fa fa-warning text-danger form-control-feedback\"\n        [hidden]=\"model.formGroup.controls[name].valid || model.formGroup.controls[name].pristine\"></span>\n    <small class=\"text-danger\"\n        [hidden]=\"model.formGroup.controls[name].valid || model.formGroup.controls[name].pristine\">\n    <span *ngFor=\"let error of errors\">{{ model.formGroup.controls[name]['messages'][error] }}</span>\n    </small>\n  </div>\n</div>\n<div>\n  <input *ngIf=\"model.type == 'hidden'\" [id]=\"name\" [formControlName]=\"name\" type=\"hidden\" />\n</div>\n",
                styles: ['input[type="checkbox"] { display: none; }',
                    'input[type="checkbox"] + label:before { font-family: FontAwesome; }',
                    'input[type="checkbox"] + label:before { content: "\\f096"; }',
                    'input[type="checkbox"]:checked + label:before { content: "\\f046"; }',
                    `
      input[type='checkbox'] + label {
        display: inline-block;
        width: 15px;
        height: 20px;
        margin: -1px 4px 0 0;
        vertical-align: middle;
        cursor: pointer;
      }
    `]
            },] }
];
EditorComponent.ctorParameters = () => [];
EditorComponent.propDecorators = {
    model: [{ type: Input }],
    name: [{ type: Input }],
    value: [{ type: Output }, { type: Input }]
};

/**
 * This component creates a complete editor from a viewmodel. The viewmodel should have
 * decorators to control the form's behavior. The form can be styled an supports validation.
 * It creates one field for each property not decorated with {@link Hidden}decorator.
 *
 * Example of usage:
 * <example-url>/#/editor</example-url>
 */
class AutoFormComponent {
    constructor() {
        /**
         * Ungrouped element will appear at the end, after all groupes.
         * If there are no groups this will be ignored. Optional.
         */
        this.ungroupedAfter = true;
        this.editors = new Array();
        this.groups = new Array();
    }
    ngOnInit() {
        // take all controls as is
        const hasDecorators = this.formGroup.__editorModel__;
        // loop through all fields
        for (const controlName in this.formGroup.controls) {
            if (!this.formGroup.controls.hasOwnProperty(controlName)) {
                continue;
            }
            let displayOrder = 0;
            let groupOrder = 0;
            let groupName;
            let groupDesc;
            let isInGroup = false;
            if (hasDecorators) {
                displayOrder = this.formGroup.__editorModel__[`__displayOrder__${controlName}`];
                isInGroup = !!this.formGroup.__editorModel__[`__isGrouped__${controlName}`];
                if (isInGroup) {
                    groupName = this.formGroup.__editorModel__[`__groupName__${controlName}`];
                    groupOrder = this.formGroup.__editorModel__[`__groupOrder__${controlName}`];
                    groupDesc = this.formGroup.__editorModel__[`__groupDesc__${controlName}`];
                }
            }
            if (!isInGroup) {
                // if not part of any group just push to main group
                this.editors.push({
                    key: displayOrder,
                    editor: controlName
                });
            }
            else {
                // check if group already exists
                const existingGroupArray = this.groups.filter((g) => g.name === groupName);
                const groupExists = existingGroupArray.length === 1;
                if (!groupExists) {
                    this.groups.push({
                        key: groupOrder,
                        name: groupName,
                        desc: groupDesc,
                        editors: new Array()
                    });
                }
                // add field to existing group (assume that here the group always exists)
                const existingGroup = this.groups.filter((g) => g.name === groupName)[0];
                // and store
                existingGroup.editors.push({
                    key: displayOrder,
                    editor: controlName
                });
            }
        }
        // check the display decorator and sort
        this.editors.sort((e, n) => e.key - n.key);
        // sort groups
        this.groups.sort((e, n) => e.key - n.key);
    }
    grouped() {
        return this.groups && this.groups.length > 0;
    }
}
AutoFormComponent.decorators = [
    { type: Component, args: [{
                selector: 'ac-autoform',
                template: "<ng-content></ng-content>\n<ng-container *ngIf=\"!ungroupedAfter\">\n  <ac-editor *ngFor=\"let editor of editors\" [name]=\"editor.editor\" [formGroup]=\"formGroup\"></ac-editor>\n</ng-container>\n<ng-container *ngIf=\"grouped()\">\n  <fieldset *ngFor=\"let group of groups\">\n    <legend [attr.title]=\"group.desc\" *ngIf=\"group.name\">{{ group.name }}</legend>\n    <ac-editor *ngFor=\"let editor of group.editors\" [name]=\"editor.editor\" [formGroup]=\"formGroup\"></ac-editor>\n  </fieldset>\n</ng-container>\n<ng-container *ngIf=\"ungroupedAfter\">\n  <ac-editor *ngFor=\"let editor of editors\" [name]=\"editor.editor\" [formGroup]=\"formGroup\"></ac-editor>\n</ng-container>\n",
                styles: ['fieldset { border-top: 1px silver solid; padding: 10px; }',
                    'legend { width: auto; padding-left: 10px; padding-right: 10px; font-size: 1em;}']
            },] }
];
AutoFormComponent.ctorParameters = () => [];
AutoFormComponent.propDecorators = {
    formGroup: [{ type: Input }],
    ungroupedAfter: [{ type: Input }]
};

/**
 * The treeview can held any tree of nodes, which can be styles, linked, and bound to events.
 * Clicking on the items fires various events.
 *
 * The component creates a treeview that can handle {@link TreeNodeModel objects that create text, checkboxes, or}highlights.
 * A node can have an icon. The icons are based on FontAwesome's css classes.
 *
 * Example of usage:
 * <example-url>/#/widget/tree</example-url>
 */
class TreeViewComponent {
    constructor() {
        this.nodeClick = new EventEmitter();
        this.checkChanged = new EventEmitter();
        this.selectedChanged = new EventEmitter();
        this.collapseChanged = new EventEmitter();
    }
    onNodeClick(node) {
        this.nodeClick.emit(node);
    }
    onCheckChanged(node) {
        this.checkChanged.emit(node);
    }
    onSelectedChanged(node) {
        this.selectedChanged.emit(node);
    }
    onCollapseChanged(node) {
        this.collapseChanged.emit(node);
    }
}
TreeViewComponent.decorators = [
    { type: Component, args: [{
                selector: 'ac-tree',
                template: "<ul class=\"treeview\">\n  <ac-treenode\n    [node]=\"nodes\"\n    (nodeClick)=\"onNodeClick($event)\"\n    (checkChanged)=\"onCheckChanged($event)\"\n    (selectedChanged)=\"onSelectedChanged($event)\"\n    (collapseChanged)=\"onCollapseChanged($event)\"\n  ></ac-treenode>\n</ul>\n",
                styles: ['.treeview { list-style: none; margin-left: -25px; }']
            },] }
];
TreeViewComponent.ctorParameters = () => [];
TreeViewComponent.propDecorators = {
    nodes: [{ type: Input }],
    nodeClick: [{ type: Output }],
    checkChanged: [{ type: Output }],
    selectedChanged: [{ type: Output }],
    collapseChanged: [{ type: Output }]
};

/**
 * The TreeNodeModel class represents a single TreeNodeModel for the treeview. Actually, this renders the real UI.
 *
 * The component needs this input value:
 *
 * * @Input() node: AcTextTreeNode;
 *
 * The several options are set through the AcTreeNodeOptions object that is being held by the
 * AcTextTreeNode class. This includes colors, icons, and other styles.
 *
 * The component can fire these events:
 *
 * * @Output() nodeClick: EventEmitter<AcTreeNode>;
 * * @Output() checkChanged: EventEmitter<AcTreeNode>;
 * * @Output() selectedChanged: EventEmitter<AcTreeNode>;
 * * @Output() collapseChanged: EventEmitter<AcTreeNode>;
 *
 */
class TreeViewNodeComponent {
    constructor(el, renderer) {
        this.el = el;
        this.renderer = renderer;
        this.isExpanded = false;
        this.nodeClick = new EventEmitter();
        this.checkChanged = new EventEmitter();
        this.selectedChanged = new EventEmitter();
        this.collapseChanged = new EventEmitter();
    }
    ngOnInit() {
        // set HTML according to options
        this.collapseClasses = new Array();
        this.iconClasses = new Array();
        // expander area with icon
        if (this.node) {
            // expect a font-awesome class with or without fa class
            if (this.node.options && this.node.options.icon) {
                if (this.node.options.icon.indexOf(`${TreeViewNodeComponent.pfxIcon} `) !== 0) {
                    this.iconClasses.push('fa');
                }
                this.iconClasses.push(this.node.options.icon);
            }
            if (this.node.options && this.node.options.href) {
                this.href = this.node.options.href;
            }
            else {
                this.href = '';
            }
            // open/close area for elements with children
            if (this.node.hasChildren) {
                this.collapseClasses.push('ac-icon'); // base class
                this.collapseClasses.push(TreeViewNodeComponent.pfxIcon);
                this.collapseClasses.push(TreeViewNodeComponent.opnIcon);
                // collapsed by default
                this.node.state &= ~TreeNodeState.checked;
            }
            if (this.node.options && this.node.options.color) {
                this.foreColor = this.node.options.color;
            }
            if (this.node.options && this.node.options.backColor) {
                this.backColor = this.node.options.backColor;
            }
            this.node.stateChange.subscribe((se) => {
                this.collapseChanged.emit(this.node);
                if (!this.node.stateIsExpandend) {
                    this.collapseClasses = this.collapseClasses.filter(s => s !== TreeViewNodeComponent.clsIcon);
                    this.collapseClasses.push(TreeViewNodeComponent.opnIcon);
                }
                else {
                    this.collapseClasses = this.collapseClasses.filter(s => s !== TreeViewNodeComponent.opnIcon);
                    this.collapseClasses.push(TreeViewNodeComponent.clsIcon);
                }
                this.isExpanded = this.node.stateIsExpandend;
            });
        }
    }
    // forward events in the node tree
    onNodeClick(node) {
        this.nodeClick.emit(node);
    }
    onCheckChanged(node) {
        this.checkChanged.emit(node);
    }
    onSelectedChanged(node) {
        this.selectedChanged.emit(node);
    }
    onCollapseChanged(node) {
        if (this.node.options.collapsable && this.node.hasChildren) {
            this.collapseChanged.emit(node);
        }
    }
    handleCheckChange() {
        if (this.node.options.checkable && !this.node.stateIsDisabled) {
            if (this.node.state & TreeNodeState.checked) {
                this.node.state &= ~TreeNodeState.checked;
            }
            else {
                this.node.state |= TreeNodeState.checked;
            }
            this.checkChanged.emit(this.node);
        }
    }
    handleClick($event) {
        $event.stopPropagation();
        // always emit click
        this.nodeClick.emit(this.node);
    }
    handleCollapse() {
        // if collapsable handle icons and view state
        if (this.node.options.collapsable) {
            // toggle state
            if (this.node.state && this.node.state & TreeNodeState.expanded) {
                this.node.state &= ~TreeNodeState.expanded;
            }
            else {
                this.node.state |= TreeNodeState.expanded;
            }
        }
    }
    handlePreSelection(state) {
        // handle the states according to options
        if (this.node.options.selectable && !this.node.stateIsDisabled) {
            if (state) {
                if (this.node.options && this.node.options.color) {
                    this.foreColor = this.node.options.backColor;
                }
                else {
                    this.foreColor = '#fff';
                }
                if (this.node.options && this.node.options.backColor) {
                    this.backColor = this.node.options.color;
                }
                else {
                    this.backColor = '#000';
                }
            }
            else {
                if (this.node.options && this.node.options.color) {
                    this.foreColor = this.node.options.color;
                }
                else {
                    this.foreColor = '#000';
                }
                if (this.node.options && this.node.options.backColor) {
                    this.backColor = this.node.options.backColor;
                }
                else {
                    this.backColor = '#fff';
                }
            }
            this.preSelectState = state;
        }
    }
    handleSelection($event) {
        // handle the states according to options
        if (this.node.options.selectable &&
            !this.node.stateIsDisabled &&
            this.preSelectState === true) {
            if (this.node.state & TreeNodeState.selected) {
                this.node.state &= ~TreeNodeState.selected;
            }
            else {
                this.node.state |= TreeNodeState.selected;
            }
            if (this.node.stateIsSelected) {
                if (this.node.options && this.node.options.color) {
                    this.foreColor = this.node.options.backColor;
                }
                else {
                    this.foreColor = '#fff';
                }
                if (this.node.options && this.node.options.backColor) {
                    this.backColor = this.node.options.color;
                }
                else {
                    this.backColor = '#000';
                }
            }
            else {
                if (this.node.options && this.node.options.color) {
                    this.foreColor = this.node.options.color;
                }
                else {
                    this.foreColor = '#000';
                }
                if (this.node.options && this.node.options.backColor) {
                    this.backColor = this.node.options.backColor;
                }
                else {
                    this.backColor = '#fff';
                }
            }
            this.selectedChanged.emit(this.node);
        }
        if (!this.href) {
            $event.preventDefault();
            $event.stopPropagation();
        }
    }
}
TreeViewNodeComponent.pfxIcon = 'fa';
TreeViewNodeComponent.opnIcon = 'fa-plus';
TreeViewNodeComponent.clsIcon = 'fa-minus';
TreeViewNodeComponent.decorators = [
    { type: Component, args: [{
                selector: 'ac-treenode',
                template: "<li class=\"treeview\" (click)=\"handleClick($event)\">\n    <i class=\"ac-collapse\" [ngClass]=\"collapseClasses\" *ngIf=\"node.hasChildren\" (click)=\"handleCollapse()\"></i>\n    <i class=\"ac-collapse\" *ngIf=\"!node.hasChildren\"></i>\n    <i class=\"ac-icon\" [ngClass]=\"iconClasses\" [style.color]=\"node.options.iconColor\" *ngIf=\"!node.options.checkable\"></i>\n    <input type=\"checkbox\" [id]=\"node.name\" *ngIf=\"node.options.checkable\" \n           [checked]=\"node.stateIsChecked\" (click)=\"handleCheckChange()\">\n    <label [attr.for]=\"node.name\"></label>\n    <a class=\"ac-container\"\n          [href]=\"href\"\n          [style.color]=\"foreColor\" \n          [style.background-color]=\"backColor\" \n          (mouseover)=\"handlePreSelection(true)\"\n          (mouseout)=\"handlePreSelection(false)\"\n          (click)=\"handleSelection($event)\">\n        {{ node.text }}\n    </a>\n    <ul class=\"treeview\" *ngIf=\"node.hasChildren\" [hidden]=\"!isExpanded\">\n        <ac-treenode *ngFor=\"let child of node.children\" \n                     [node]=\"child\"\n                     (nodeClick)=\"onNodeClick($event)\" \n                     (checkChanged)=\"onCheckChanged($event)\"\n                     (selectedChanged)=\"onSelectedChanged($event)\"\n                     (collapseChanged)=\"onCollapseChanged($event)\">\n        </ac-treenode>\n    </ul>                   \n</li>",
                styles: ["ul.treeview{list-style:none;margin-left:-2em}li.treeview{box-sizing:border-box;margin-bottom:3px;margin-left:10px}li.treeview a.ac-container{border-radius:2px;display:inline-block;padding:3px;text-decoration:none}li.treeview input[type=checkbox]{display:none}li.treeview input[type=checkbox]+label:before{content:\"\\f096\";font-family:FontAwesome}li.treeview input[type=checkbox]:checked+label:before{content:\"\\f046\"}li.treeview input[type=checkbox]+label{cursor:pointer;display:inline-block;height:20px;margin:-1px 4px 0 0;vertical-align:middle;width:15px}li.treeview i.ac-collapse{margin-left:-1.7em}li.treeview i.ac-collapse,li.treeview i.ac-icon{cursor:pointer;display:inline-block;width:15px}li.treeview .ac-node-disabled{color:silver;cursor:not-allowed}"]
            },] }
];
TreeViewNodeComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 }
];
TreeViewNodeComponent.propDecorators = {
    node: [{ type: Input }],
    nodeClick: [{ type: Output }],
    checkChanged: [{ type: Output }],
    selectedChanged: [{ type: Output }],
    collapseChanged: [{ type: Output }]
};

// import { FormValidatorFromJsonService } from './services/formvalidator-fromjson.service';
const SVOGV_COMPONENTS = [
    DataGridComponent,
    DataGridPaginationComponent,
    TreeViewComponent,
    TreeViewNodeComponent,
    EditorComponent,
    AutoFormComponent
];
const provider = [SvogvCoreModule];
/**
 * The actual SVOGV Module definition.
 */
class SvogvBootstrapModule {
    static forRoot() {
        return {
            ngModule: SvogvBootstrapModule,
            providers: provider
        };
    }
}
SvogvBootstrapModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, ReactiveFormsModule, FormsModule],
                declarations: [...SVOGV_COMPONENTS, FormatDataPipe],
                providers: provider,
                exports: SVOGV_COMPONENTS
            },] }
];

/*
 * Public API Surface of bootstrap-ui
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AutoFormComponent, DataGridComponent, DataGridPaginationComponent, EditorComponent, FormatDataPipe, SvogvBootstrapModule, TreeViewComponent, TreeViewNodeComponent };
//# sourceMappingURL=svogv-bootstrap.js.map
